import json
import logging
import sys

import arrow

__all__ = ["SmkLog"]


class SmkLog(object):

    def __init__(self, platform: str, service: str):
        # e.g. youzan, or 'storefront_wechat' etc. for our own platform
        self.platform = platform

        # e.g. nomad_envoy_order or storefront_auth
        self.service = service

        self._logger = get_log("%s_%s" % (self.platform, self.service))

    """
    All these different log level functions will be overwrote with full
    signature and implementation in the end of current file.
    """

    def info(self, *args, **kw) -> str:
        pass

    def debug(self, *args, **kw) -> str:
        pass

    def error(self, *args, **kw) -> str:
        pass

    def warn(self, *args, **kw) -> str:
        pass

    def critical(self, *args, **kw) -> str:
        pass


def json_dumps_unicode(obj):
    """
    Change Python default json.dumps acting like JavaScript, including allow
    Chinese characters and no space between any keys or values.
    """
    return json.dumps(obj,
                      ensure_ascii=False,
                      separators=(',', ':')
                      )


def get_log(name):
    log = logging.getLogger(name)
    log.setLevel(logging.INFO)

    # ref: https://stackoverflow.com/questions/19561058/duplicate-output-in-simple-python-logging-configuration  # noqa:E501
    log.propagate = False  # stop producing duplicated logs

    if not log.handlers:
        out_handler = logging.StreamHandler(sys.stdout)
        out_handler.setFormatter(logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
        out_handler.setLevel(logging.INFO)
        log.addHandler(out_handler)

    return log


for level in ["info", "debug", "error", "warn", "critical"]:
    def closure_log(level):
        def log(self,
                source: str,
                category: str,
                label: str,
                message: str,
                data: object,
                timing: object,
                user: str,
                store_id: str = None,
                request_id: str = None,
                timestamp: str = None) -> str:
            if isinstance(data, (list, dict)):
                data = json_dumps_unicode(data)
            else:
                data = str(data)

            if timestamp is None:
                timestamp = arrow.now().isoformat()  # including zone info.

            msg = {
                "platform": self.platform,
                "service": self.service,

                "source": source,
                "category": category,
                "label": label,
                "message": message,
                "data": data,
                "timing": timing,

                "user": user,
                "timestamp": timestamp,
            }
            if store_id:
                msg.update({"store_id": store_id})
            if request_id:
                msg.update({"request_id": request_id})
            msg = json_dumps_unicode(msg)

            getattr(self._logger, level)(msg)  # log current message.

            return msg

        return log


    setattr(SmkLog, level, closure_log(level))



from os import path

from setuptools import setup, find_packages  # noqa: H301

NAME = "smklog"
VERSION = "0.1.5"

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["arrow"]

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name=NAME,
    version=VERSION,
    description="Samarkand logger",
    author="David Chen",
    author_email="david.chen@samarkand.global",
    url="https://gitlab.com/samarkand-util/smklog",
    keywords=["logger"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description=long_description,
    long_description_content_type='text/markdown',
)



import json
import unittest

from smklog import SmkLog


class TestSmkLog(unittest.TestCase):
    platform = "youzan"
    service = "nomad_envoy_order"
    maxDiff = None

    def test_smklog(self):
        logger = SmkLog(self.platform, self.service)

        source = "odoo"
        category = "info"
        label = "get_all_orders"

        message = "Translated error message for youzan service not responding"

        data = """{"key":"value","键":"值"}"""

        timing = """{"total":3}"""

        user = "1234567"

        store_id = "samarkand.youzan.foreveryoung"
        request_id = "0EFD81C9-16BF-4FF6-8CF8-00FEB0619A47"

        msg1 = logger.info(source, category, label, message, data, timing, user)
        self.assertTrue(isinstance(json.loads(msg1), dict),
                        "Message should be JSON format")
        self.assertEqual(json.loads(msg1)["data"], data,
                         "Chinese characters should keep original, not escaped")
        self.assertFalse("store_id" in json.loads(msg1),
                         "store_id is optional.")
        self.assertFalse("request_id" in json.loads(msg1),
                         "request_id is also optional")

        msg2 = logger.info(source, category, label, message, data, timing, user,
                           store_id, request_id)

        include_field_msg = "all fields should be included, including store_id and request_id."
        self.assertIn("store_id", json.loads(msg2), include_field_msg)
        self.assertIn("request_id", json.loads(msg2), include_field_msg)

        self.assertTrue("store_id" in json.loads(msg2),
                        "we passed store_id field")
        self.assertTrue("request_id" in json.loads(msg2),
                        "we passed request_id field")

        data_json = {"key": "value", "键": "值"}
        msg3 = logger.error(source, category, label, message, data_json,
                            timing, user, store_id, request_id)
        self.assertEqual(json.loads(msg3)["data"], '{"key":"value","键":"值"}',
                         "Chinese characters should keep original even when "
                         "data is JSON data format.")
